@@ -2,7 +2,7 @@
 
 ;; Copyright (C) 2015  Jorgen Schaefer <contact@jorgenschaefer.de>
 
-;; Version: 1.5
+;; Version: 1.7
 ;; Author: Jorgen Schaefer <contact@jorgenschaefer.de>
 
 ;; This program is free software; you can redistribute it and/or
@@ -161,6 +161,12 @@ MATCHER is either a matcher defined with
       (cons t (format "Expected %S not to `equal' %S" a b))
     (cons nil (format "Expected %S to `equal' %S" a b))))
 
+(buttercup-define-matcher :to-have-same-items-as (a b)
+  (if (and (cl-subsetp a b :test #'equal)
+           (cl-subsetp b a :test #'equal))
+      (cons t (format "Expected %S not to have same items as %S" a b))
+    (cons nil (format "Expected %S to have same items as %S" a b))))
+
 (buttercup-define-matcher :to-match (text regexp)
   (if (string-match regexp text)
       (cons t (format "Expected %S not to match the regexp %S"
@@ -558,7 +564,7 @@ KEYWORD can have one of the following values:
 
 (defvar buttercup--cleanup-functions nil)
 
-(defmacro buttercup--with-cleanup (&rest body)
+(defmacro buttercup-with-cleanup (&rest body)
   `(let ((buttercup--cleanup-functions nil))
      (unwind-protect (progn ,@body)
        (dolist (fun buttercup--cleanup-functions)
@@ -671,6 +677,9 @@ current directory."
           (error "Option requires argument: %s" (car args)))
         (push (cadr args) patterns)
         (setq args (cddr args)))
+       ((member (car args) '("-c" "--no-color"))
+        (setq buttercup-color nil)
+        (setq args (cdr args)))
        (t
         (push (car args) dirs)
         (setq args (cdr args)))))
@@ -678,8 +687,11 @@ current directory."
     (dolist (dir (or dirs '(".")))
       (dolist (file (directory-files-recursively
                      dir "\\`test-.*\\.el\\'\\|-tests?\\.el\\'"))
-        (when (not (string-match "/\\." (file-relative-name file)))
-          (load file nil t))))
+        (when (not (string-match "\\(^\\|/\\)\\." (file-relative-name file)))
+          (condition-case err
+              (load file nil t)
+            (error
+             (message (format "Error while loading %s:\n%s" file (error-message-string err))))))))
     (when patterns
       (let ((suites-or-specs buttercup-suites))
         (while suites-or-specs
@@ -753,7 +765,7 @@ Do not change the global value.")
 
 (defun buttercup--run-spec (spec)
   (funcall buttercup-reporter 'spec-started spec)
-  (buttercup--with-cleanup
+  (buttercup-with-cleanup
    (dolist (f buttercup--before-each)
      (buttercup--update-with-funcall spec f))
    (buttercup--update-with-funcall spec (buttercup-spec-function spec))
@@ -815,13 +827,18 @@ suite-done -- A suite has finished. The argument is the spec.
 
 buttercup-done -- All suites have run, the test run is over.")
 
+(defvar buttercup-color t
+  "Whether to use colors in output.")
+
 (defun buttercup-reporter-adaptive (event arg)
   "A reporter that handles both interactive and noninteractive sessions.
 
 Calls either `buttercup-reporter-batch' or
 `buttercup-reporter-interactive', depending."
   (if noninteractive
-      (buttercup-reporter-batch event arg)
+      (if buttercup-color
+          (buttercup-reporter-batch-color event arg)
+        (buttercup-reporter-batch event arg))
     (buttercup-reporter-interactive event arg)))
 
 (defvar buttercup-reporter-batch--start-time nil
@@ -921,9 +938,107 @@ Calls either `buttercup-reporter-batch' or
       (_
        (error "Unknown event %s" event)))))
 
+(defun buttercup-reporter-batch-color (event arg)
+  (pcase event
+    (`spec-done
+     (let ((level (length (buttercup-spec-parents arg))))
+       (cond
+        ((eq (buttercup-spec-status arg) 'passed)
+         (buttercup--print (buttercup-colorize "\r%s%s\n" 'green)
+                           (make-string (* 2 level) ?\s)
+                           (buttercup-spec-description arg)))
+      ((eq (buttercup-spec-status arg) 'failed)
+       (buttercup--print (buttercup-colorize "\r%s%s  FAILED\n" 'red)
+                         (make-string (* 2 level) ?\s)
+                         (buttercup-spec-description arg))
+       (setq buttercup-reporter-batch--failures
+             (append buttercup-reporter-batch--failures
+                     (list arg))))
+      ((eq (buttercup-spec-status arg) 'pending)
+       (if (equal (buttercup-spec-failure-description arg) "SKIPPED")
+           (buttercup--print "  %s\n" (buttercup-spec-failure-description arg))
+         (buttercup--print (buttercup-colorize "\r%s%s  %s\n" 'yellow)
+                           (make-string (* 2 level) ?\s)
+                           (buttercup-spec-description arg)
+                           (buttercup-spec-failure-description arg))))
+      (_
+       (error "Unknown spec status %s" (buttercup-spec-status arg))))))
+
+    (`buttercup-done
+     (dolist (failed buttercup-reporter-batch--failures)
+       (let ((description (buttercup-spec-failure-description failed))
+             (stack (buttercup-spec-failure-stack failed)))
+         (buttercup--print "%s\n" (make-string 40 ?=))
+         (buttercup--print (buttercup-colorize "%s\n" 'red) (buttercup-spec-full-name failed))
+         (when stack
+           (buttercup--print "\nTraceback (most recent call last):\n")
+           (dolist (frame stack)
+             (let ((line (format "  %S" (cdr frame))))
+               (when (> (length line) 79)
+                 (setq line (concat (substring line 0 76)
+                                    "...")))
+               (buttercup--print "%s\n" line))))
+         (cond
+          ((stringp description)
+           (buttercup--print (concat (buttercup-colorize "FAILED" 'red ) ": %s\n")
+                             description))
+          ((eq (car description) 'error)
+           (buttercup--print "%S: %S\n\n"
+                             (car description)
+                             (cadr description)))
+          (t
+           (buttercup--print "FAILED: %S\n" description)))
+         (buttercup--print "\n")))
+     (let ((defined (buttercup-suites-total-specs-defined arg))
+           (pending (buttercup-suites-total-specs-pending arg))
+           (failed (buttercup-suites-total-specs-failed arg))
+           (duration (- (float-time)
+                        buttercup-reporter-batch--start-time)))
+       (if (> pending 0)
+           (buttercup--print
+            (concat
+             "Ran %s out of %s specs,"
+             (buttercup-colorize " %s failed" (if (eq 0 failed) 'green 'red))
+             ", in %.1f seconds.\n")
+            (- defined pending)
+            defined
+            failed
+            duration)
+         (buttercup--print
+          (concat
+           "Ran %s specs,"
+           (buttercup-colorize " %s failed" (if (eq 0 failed) 'green 'red))
+           ", in %.1f seconds.\n")
+          defined
+          failed
+          duration))
+       (when (> failed 0)
+         (error ""))))
+
+    (_
+     ;; Fall through to buttercup-reporter-batch implementation.
+     (buttercup-reporter-batch event arg)))
+  )
+
 (defun buttercup--print (fmt &rest args)
   (send-string-to-terminal (apply #'format fmt args)))
 
+(defconst buttercup-colors
+  '((black   . 30)
+    (red     . 31)
+    (green   . 32)
+    (yellow  . 33)
+    (blue    . 34)
+    (magenta . 35)
+    (cyan    . 36)
+    (white   . 37))
+  "List of text colors.")
+
+(defun buttercup-colorize (string color)
+  "Format STRING with COLOR."
+  (let ((color-code (cdr (assoc color buttercup-colors))))
+    (format "\u001b[%sm%s\u001b[0m" color-code string)))
+
 (defun buttercup-reporter-interactive (event arg)
   "Reporter for interactive uses."
   ;; This is a bit rudimentary ...
@@ -995,5 +1110,29 @@ failed -- The second value is the description of the expectation
             frame (backtrace-frame n)))
     frame-list))
 
+;;;###autoload
+(define-minor-mode buttercup-minor-mode
+  "Activate buttercup minor mode.
+
+With buttercup minor mode active the following is activated:
+
+- `describe' and `it' forms are fontified with
+  `font-lock-keyword-face'.
+- `describe' and `it' forms are available from `imenu' for
+  quicker access."
+  :lighter " ❀"
+  (let ((font-lock-form '(("(\\(describe\\|buttercup-define-matcher\\|it\\) "
+                           1 'font-lock-keyword-face)))
+        (imenu-forms '(("Test Suites" "\\((describe\\_> +\\)\"\\(\\_<.+\\_>\\)\"" 2)
+                       ("Spec" "\\((it\\_> +\\)\"\\(\\_<.+\\_>\\)\"" 2))))
+    (if buttercup-minor-mode
+        (progn
+          (font-lock-add-keywords nil font-lock-form)
+          (cl-dolist (form imenu-forms)
+            (add-to-list 'imenu-generic-expression form)))
+      (font-lock-remove-keywords nil font-lock-form)
+      (cl-dolist (form imenu-forms)
+        (setq imenu-generic-expression (delete form imenu-generic-expression))))))
+
 (provide 'buttercup)
 ;;; buttercup.el ends here
